#!/bin/perl

use strict;
package acidrip_dvdinfo;

use IO::Handle;
use POSIX;
use Data::Dumper;

my $name = "AcidRip";

#
# dvd+->track[#]------+->chapter[#]->stuff
#    |        ^       |->length
#    |        |     +-+->audio[#]->stuff
#    |      ttn[#]  | |
#    |        ^     +-+->subp[#]->more_stuff
#    |        |     |
#    +->vts[#]+-----+
#

#############################
###### CONSTRUCTORSISH ######
#############################

sub new_track {
	return {	'length' => '', 'vts' => '', 'chapter' => [], 'audio' => [], 'subp' => [] };
}

sub new_chapter {
	return {	'length' => '', 'cell' => 0 	};
}

sub new_cell {
	return {	'length' => '', 'chapter' => 0 	};
}

sub new_audio {
	return {	'lang_code' => '',	'language'  => '',	'channels'  =>  0,	'format' => '',	'extension' => '',	'quantization' => ''	};
}

sub new_subp {
	return {	'lang_code' => '',	'language'  => '',	'extension' => ''	};
}

sub new_vts {	
	return {	'audio' => [],	'subp'  => [],	'angle' => [],	'format' => '', 'ttn'   => []	, 'width' => 0, 'height' => 0, 'aspect' => 0, 'fps' => 0 };
}

sub new
{
	my $class = shift;
	my $dvd = { 'track' => [], 'vts' => [], 'valid' => 'no', 'title' => 'unknown'	};
	return bless ($dvd, $class);
}

###############################################
###### SECRET LEVEL SUPER BONUS ROUTINES ######
###############################################

sub find_crop ($$) {
	my $dvd = shift;
	my $crop_output;
	if ($dvd->{'valid'} eq 'yes')
	{
    my $track = shift || die "title number not found for crop\n";
    my $length = seconds($dvd->get_length($track));
    my $frames = 10;
		my $dvd_device = $::settings->{'$dvd_device'};
    my $sstep = int $length / (3 * $frames) + 1;
    my %crop;
	  open (CROP, "$::settings->{'mplayer'} -vop cropdetect -dvd $track -dvd-device $::settings->{'dvd_device'} -sstep $sstep -nosound -vo null -frames $frames 2>&1 |");
    while (<CROP>){ 
			if ($_ =~ /crop area.*\(-vop (crop=\d*:\d*:\d*:\d*)/) { $crop{$1}++ }
			$crop_output .= $_;
		}
	  my @order = sort { $crop{$b} <=> $crop{$a} } keys %crop;
		close CROP;
    return ( $order[0], $crop_output )
  } else { return ( -1, "crop detect failed" ) }
}

############################
###### BONUS ROUTINES ######
############################

# Convert an hh:mm:ss string to seconds
sub seconds ($)
{
  my $secs;
  my $hhmmss = shift; # || die "seconds not provided\n";
  if ($hhmmss =~ /(\d\d):(\d\d):(\d\d)/) {
    $secs = ((($1*60)+$2)*60)+$3;
  } else { print "malformed hh:mm:ss values! $hhmmss\n"};
  return $secs;
}

# Return length of a given dvd title
sub get_length ($$)
{
  my $dvd = shift;
  my $this_track = shift;
  return $dvd->{'track'}[$this_track]->{'length'};
}

sub get_selection_length
{
	my ($dvd, $track, $start, $end) = @_;
	my $length = 0;
	for my $chapter ($start..$end) {
		$length += seconds($dvd->{'track'}[$track]->{'chapter'}[$chapter]->{'length'});
	}
	return $length;
}

# Find longest title on a DVD
sub get_main_title ($)
{
  my $dvd = shift;
  my $max = 0;
  my $main = 0;
	my $nr_of_tracks = scalar @{$dvd->{'track'}} -1 ;
  for my $t (1..$nr_of_tracks)
  {
    if ($max < seconds($dvd->get_length($t))) {
      $main = $t;
      $max = seconds($dvd->get_length($t));
    }
  }
  return $main;
}

###########################
###### CORE ROUTINES ######
###########################

sub read_disc
{
	my $dvd = shift;
	my $lsdvd = shift;
	my $dvd_device = shift;
	my $this_track;
	my $this_audio;
	my $this_chapter;
	my $this_vts;
	my $lsdvd_output;

	$dvd->{'valid'} = 'no';
	$dvd->{'title'} = 'unknown';
	
	$dvd->{'track'} = [];
	$dvd->{'vts'} = [];
	
	open(LSDVD, "$lsdvd -x $dvd_device 2>&1 |") || (return "lsdvd failed. is it installed?");
	while (<LSDVD>)
	{
		$lsdvd_output .= $_;

		if (/Disc Title:\s(\w+)/) {
			$dvd->{'title'} = $1;
		}
			
		if (/Title:\s(\d\d).+\sLength:\s(\d\d:\d\d:\d\d)/) { 
			$this_track = $1;
			$dvd->{'track'}[$this_track] = new_track;
			$dvd->{'track'}[$this_track]->{'length'} = $2;
			$dvd->{'valid'} = 'yes';
		}

		if (/VTS:\s(\d+),.+TTN:\s(\d+),/) {
			$this_vts = $1;
			$dvd->{'vts'}[$this_vts] = new_vts;
			$dvd->{'track'}[$this_track]->{'audio'} = $dvd->{'vts'}[$this_vts]->{'audio'};
			$dvd->{'track'}[$this_track]->{'subp'} = $dvd->{'vts'}[$this_vts]->{'subp'};
			$dvd->{'track'}[$this_track]->{'vts'} = $dvd->{'vts'}[$this_vts];
		}
		
		if (/FPS:\s([0-9.]+),.+Format:\s(\w+)/) {
			$dvd->{'track'}[$this_track]->{'vts'}->{'fps'} = $1;
			$dvd->{'track'}[$this_track]->{'vts'}->{'format'} = $2;
		}

		if (/Aspect\sratio:\s([0-9\/]+).+Width:\s(\d+),.+Height:\s(\d+)/) {
			$dvd->{'track'}[$this_track]->{'vts'}->{'aspect'} = $1;
			$dvd->{'track'}[$this_track]->{'vts'}->{'width'} = $2;
			$dvd->{'track'}[$this_track]->{'vts'}->{'height'} = $3;
		}

		if (/Chapter:\s(\d\d),\sLength:\s(\d\d:\d\d:\d\d),\sStart\sCell:\s(\d\d)/) {
		  $dvd->{'track'}[$this_track]->{'chapter'}[$1] = new_chapter;
			$dvd->{'track'}[$this_track]->{'chapter'}[$1]->{'length'} = $2;
			$dvd->{'track'}[$this_track]->{'chapter'}[$1]->{'cell'} = $3;
			$dvd->{'track'}[$this_track]->{'cell'}[$3] = new_cell;
			$dvd->{'track'}[$this_track]->{'cell'}[$3]->{'chapter'} = $1;
		}
		
		if (/Cell:\s(\d\d),\sLength:\s(\d\d:\d\d:\d\d)/) {
			if ( ! defined $dvd->{'track'}[$this_track]->{'cell'}[$1] ) {
				
				$dvd->{'track'}[$this_track]->{'cell'}[$1] = new_cell;
				$dvd->{'track'}[$this_track]->{'cell'}[$1]->{'chapter'} = $dvd->{'track'}[$this_track]->{'cell'}[$1 - 1]->{'chapter'};
			}
			$dvd->{'track'}[$this_track]->{'cell'}[$1]->{'length'} = $2;

		}

		if (/Audio:\s(\d)/) {
			$this_audio = $1; 
			if ( ! defined $dvd->{'vts'}[$this_vts]->{'audio'}[$1]) {

				$dvd->{'track'}[$this_track]->{'audio'}[$1] = new_audio; 

				if (/Audio.+Language:\s(\w\w)\s-\s(\w+),\s.+Content:\s(\w+)/) {
					$dvd->{'track'}[$this_track]->{'audio'}[$this_audio]->{'lang_code'} = $1;
					$dvd->{'track'}[$this_track]->{'audio'}[$this_audio]->{'language'} = $2;
					$dvd->{'track'}[$this_track]->{'audio'}[$this_audio]->{'extension'} = $3;
				}
				if (/Audio.+Format:\s(\w+),.+Frequency:\s(\w+)/) {
					$dvd->{'track'}[$this_track]->{'audio'}[$this_audio]->{'format'} = $1;
					$dvd->{'track'}[$this_track]->{'audio'}[$this_audio]->{'frequency'} = $2;
				}
				if (/Audio.+Quantization:\s(\w+),.+Channels:\s(\d),/) {
					$dvd->{'track'}[$this_track]->{'audio'}[$this_audio]->{'quantization'} = $1;
					$dvd->{'track'}[$this_track]->{'audio'}[$this_audio]->{'channels'} = $2;
				}
			}
		}

		if (/Subtitle:\s(\d\d),.+Language:\s(\w\w)\s-\s(\w+),.+Content:\s(\w+)/) {
			if ( ! defined $dvd->{'vts'}[$this_vts]->{'subp'}[$1]) 
			{
				$dvd->{'track'}[$this_track]->{'subp'}[$1] = new_subp;
				$dvd->{'track'}[$this_track]->{'subp'}[$1]->{'lang_code'} = $2;
				$dvd->{'track'}[$this_track]->{'subp'}[$1]->{'language'} = $3;
				$dvd->{'track'}[$this_track]->{'subp'}[$1]->{'extension'} = $4;
			}
		}
	}	
	close LSDVD;
	return $lsdvd_output;
}
 
1;
