#!/usr/bin/perl -w

package acidrip;
require 5.005;
use strict;

use AcidRip::acidrip;
use Data::Dumper;

sub quit_acidrip {Gtk->exit(0)}

sub on_select_audio
{
	my ($widget, $this_audio) = @_;
	$::settings->{'selected_audio'} = $this_audio;
	if ($this_audio > 0 ) {
		my $audio = $::dvd->{'track'}[$::settings->{'selected_track'}]->{'audio'}[$this_audio];
		my $out = "selected_audio changed to $this_audio - Channels $audio->{'channels'} frequency $audio->{'frequency'} quantization $audio->{'quantization'}";
		set_bitrate();
		message($out);
	}
}

sub on_select_subp
{
	my ($widget, $this_subp) = @_;
	$::settings->{'selected_subp'} = $this_subp;
	message("selected_subp changed to $::settings->{'selected_subp'}");
}

sub on_select_track
{
	my ($widget, $item) = @_;
	my $newitem = $widget->child_position($item) + 1;
	my $olditem = $::settings->{'selected_track'};
	if ($olditem != $newitem) {
		$::widgets->{'track_tree'}->unselect_item($olditem - 1);
		$::settings->{'selected_chapters_start'} = 0;
		$::settings->{'selected_chapters_end'} = 0;
		$::widgets->{'selected_track_label'}->set_text("Selected Track $newitem - " . $::dvd->get_length($newitem));
	}

	#load the correct audio entries for the combo
	$::widgets->{'audio_track_combo'}->list->clear_items(0,-1);
	my @audio = [];
	$audio[0] = new_with_label Gtk::ListItem("<Default> $::settings->{'language'}");
	$audio[0]->signal_connect('select', \&on_select_audio, -1);
	$::widgets->{'audio_track_combo'}->list->append_items($audio[0]);
	$audio[0]->show;

	$audio[1] = new_with_label Gtk::ListItem("<None>");
	$audio[1]->signal_connect('select', \&on_select_audio, -2);
	$::widgets->{'audio_track_combo'}->list->append_items($audio[1]);
	$audio[1]->show;

	for my $this_audio (1 .. scalar @{$::dvd->{'track'}[$newitem]->{'audio'}}-1)	{
		my $label = $this_audio . ": " . $::dvd->{'track'}[$newitem]->{'audio'}[$this_audio]->{'language'};
		if ( $::dvd->{'track'}[$newitem]->{'audio'}[$this_audio]->{'extension'} ne "Undefined" ) {
			$label = $label . ": " . $::dvd->{'track'}[$newitem]->{'audio'}[$this_audio]->{'extension'}
		}
		$audio[$this_audio + 1] = new_with_label Gtk::ListItem($label);
	  $audio[$this_audio + 1]->signal_connect('select', \&on_select_audio, $this_audio);
		$::widgets->{'audio_track_combo'}->list->append_items($audio[$this_audio+1]);
		$audio[$this_audio+1]->show;
	}

	$::widgets->{'subp_combo'}->list->clear_items(0, -1);
	my @subp = [];	
	$subp[0] = new_with_label Gtk::ListItem("<None>");
	$subp[0]->signal_connect('select', \&on_select_subp, -1);
	$::widgets->{'subp_combo'}->list->append_items($subp[0]);
	$subp[0]->show;

	for my $this_subp (1 .. scalar @{$::dvd->{'track'}[$newitem]->{'subp'}}-1)	{
	  my $label = $this_subp . ": " . $::dvd->{'track'}[$newitem]->{'subp'}[$this_subp]->{'language'};
		if ( $::dvd->{'track'}[$newitem]->{'subp'}[$this_subp]->{'extension'} ne "Undefined" ) {
			$label = $label . ": " . $::dvd->{'track'}[$newitem]->{'subp'}[$this_subp]->{'extension'}
		}
		$subp[$this_subp] = new_with_label Gtk::ListItem($label);
	  $subp[$this_subp]->signal_connect('select', \&on_select_subp, $this_subp - 1);
		$::widgets->{subp_combo}->list->append_items($subp[$this_subp]);
		$subp[$this_subp]->show;
	}
	
	$::settings->{'selected_track'} = $newitem;
	my $vts = $::dvd->{'track'}[$::settings->{'selected_track'}]->{'vts'};
	message("Selected Track $::settings->{'selected_track'} - Aspect: $vts->{'aspect'}, Format: $vts->{'format'}, " .
					"Size: $vts->{'width'}x$vts->{'height'}");
	set_bitrate();
}

sub on_audio_select
{
	my ($widget, $item) = @_;
}

sub select_chapters
{
	$::settings->{'busy'} = 1;
	my $chapters = scalar @{$::dvd->{'track'}[$::settings->{'selected_track'}]->{'chapter'}} - 1;
	$::widgets->{'selected_track_label'}->set_text("Selected Track $::settings->{'selected_track'} ".
		"Ch $::settings->{'selected_chapters_start'}-$::settings->{'selected_chapters_end'} - ".
		hhmmss($::dvd->get_selection_length($::settings->{'selected_track'}, $::settings->{'selected_chapters_start'},
						$::settings->{'selected_chapters_end'})));
	for my $item (1..$chapters)
	{	
		if ($item >= $::settings->{'selected_chapters_start'} && $item <= $::settings->{'selected_chapters_end'}) {
		  $::widgets->{'chapter_items'}[$::settings->{'selected_track'}][$item]->select
		} else {
			$::widgets->{'chapter_items'}[$::settings->{'selected_track'}][$item]->deselect
		}
	}
	$::settings->{'busy'} = 0;
	split_track() if $::settings->{'total_blocks'} > 1;
	set_bitrate();
}

sub on_chapter_select
{
	if (! $::settings->{'busy'}) 
	{
		my ($widget, $chapter, $track) = @_;
		if ($track != $::settings->{'selected_track'}) { $::widgets->{'track_tree'}->select_item($track - 1 )}
		my $start = $::settings->{'selected_chapters_start'};
		my $end = $::settings->{'selected_chapters_end'};
		if ($start == 0 || $chapter < $start) { $::settings->{'selected_chapters_start'} = $chapter }
		if ($end == 0 || $chapter > $end) { $::settings->{'selected_chapters_end'} = $chapter }
		select_chapters();
	}
}

sub on_chapter_deselect
{
	if (! $::settings->{'busy'}) 
	{
		my ($widget, $chapter) = @_;
		my $start = $::settings->{'selected_chapters_start'};
		my $end = $::settings->{'selected_chapters_end'};
	
		if ($end - $chapter > $chapter - $start) {
			$::settings->{'selected_chapters_start'}=$chapter;
			$::settings->{'selected_chapters_end'}=$chapter if ($start = $chapter);
		} else {
			$::settings->{'selected_chapters_end'}=$chapter;
			$::settings->{'selected_chapters_start'}=$chapter if ($end = $chapter);
		}
		select_chapters();
	}
}

sub on_video_bitrate_lock_check_clicked
{
	$::widgets->{'video_bitrate_entry'}->set_editable(! shift->get_active);
}

sub on_scale_lock_check_clicked
{
	my $widget = shift;
	$::widgets->{'scale_height_spin'}->set_sensitive(! $widget->get_active);
	if ($widget->get_active) {
		$::widgets->{'scale_height_spin'}->hide;
		$::widgets->{'scale_height_estimate_entry'}->show;
		$::widgets->{'scale_height_spin'}->set_value("-2");
	} else {
		$::widgets->{'scale_height_spin'}->show;
		$::widgets->{'scale_height_estimate_entry'}->hide;
		$::widgets->{'scale_height_spin'}->set_value($::widgets->{'scale_height_estimate_entry'}->get_text);
	}
}

sub on_filesize_changed
{
	my $free = 0;
	my $size = $::settings->{'filesize'} * $::settings->{'total_blocks'};
	open(DF, "df $::settings->{'directory'} |") if -w  $::settings->{'directory'} ? 1 : 0;
	return if -w  $::settings->{'directory'} ? 0 : 1;
	while( <DF> ) {
		my @array = split /\s+/;
		$free = $array[3] / 1024 if $. != 1;
	} close DF;
	$free = sprintf ("%d", $free);
		set_warning_text($::widgets->{'filesize_spin'}, $free <= $size ? 1 : 0);
		my $warn_block = $::settings->{'total_blocks'} > 1 ? 1 : 0;
		set_warning_text($::widgets->{'total_blocks_spin'}, $free <= $size ? $warn_block : 0);
		$::settings->{'enough_space'} = $free <= $size ? 0 : 1;
}

sub on_bpp_changed
{
	my $widget = shift;
	my $bpp = $widget->get_text;
	my $message = "Bits per pixel is "; 
	my $warn;
	
	if		($bpp < 0.10) { $message .= "FAR too low!!, increase file size or reduce resolution";	$warn = 1 }
	elsif ($bpp < 0.15) {	$message .= "quite low, output will probably suck, increase filesize or lower resolution";	$warn = 0.5}
	elsif ($bpp < 0.20) {	$message .= "alright, output will be a bit blocky, but generally ok."; $warn = 0}
	elsif ($bpp < 0.25) {	$message .= "super, output should look pretty good"; $warn = 0}
	elsif ($bpp < 0.30) {	$message .= "quite high, not much to gain here. might want a slightly higher resolution"; $warn = 0.5}
	else 								{ $message .= "very high, you'll waste the space, try a bigger resolution"; $warn = 1}
		
	set_warning_text($widget, $warn);
	
	$::widgets->{'tooltips'}->set_tip($widget, $message, "");
}

sub on_crop_detect_button_clicked
{	
	message("Crop detection in progress... please wait");

	while (Gtk->events_pending()) {Gtk->main_iteration()};

	my $crop="fail"; my $crop_output;
	($crop, $crop_output) = $::dvd->find_crop($::settings->{'selected_track'});
	message ($crop_output);
	
	if (defined $crop && $crop=~/crop=(\d+):(\d+):(\d+):(\d+)/)
	{  
  	set_setting('crop_width', $1);
		set_setting('crop_height', $2);
		set_setting('crop_horizontal', $3);
		set_setting('crop_vertical', $4);
		message('Crop detection completed');
	} else { message('Crop detection failed, is the DVD loaded?') }
}

sub on_crop_preview_button_clicked
{	
	if ($::dvd->{'valid'})
	{
		#fork proofing
		open STDIN,  '/dev/null' or message("Can't read /dev/null: $!. May fail if in background");
		
		message("Preview command: ");
		message("Previewing crop window");
	  while (Gtk->events_pending()) {Gtk->main_iteration()};
   	
		my $mplayer_output = '';
		
		my $preview = get_command("preview"); 
		
		message("Preview command: $preview");
		message("Previewing crop window");
			
		open (PREVIEW, "$preview 2>&1 |");
    while(<PREVIEW>) {
			$mplayer_output .= $_ if $_!~/rectangle/;
			while (Gtk->events_pending()) {Gtk->main_iteration()}
		}
		close PREVIEW;
	  message($mplayer_output);
		message("Preview finished");
	} else {message("Preview failed, is the DVD loaded?")}
}

sub on_setting_finished
{
	my $widget = shift;
	my $name = $::widgets->get_widget_name($widget);
	$name=~/([\w_]+)_\w+/;
	my $data=$1;
	my $ref = ref $widget;
	my $value;
	
	if ($ref eq "Gtk::Entry") {$value = $widget->get_text}
	elsif ($ref eq "Gtk::SpinButton") {$value = $widget->get_value_as_int}
	elsif ($ref eq "Gtk::CheckButton" || $ref eq "Gtk::ToggleButton") {if ($widget->get_active) {$value = 1} else {$value = 0}}
	$::settings->{$data} = $value;

	message("$data has been set to $value");
	
	if ($widget eq $::widgets->{'audio_codec_entry'}) { message($::settings->{'available_audio'}{$value}) }
	if ($widget eq $::widgets->{'video_codec_entry'}) { message($::settings->{'available_video'}{$value}) }

	on_setting_changed($widget);
}

sub on_setting_changed
{
	my $widget = shift;
	my $name = $::widgets->get_widget_name($widget);
	$name=~/([\w_]+)_\w+/;
	my $data=$1;
	my $ref = ref $widget;
	my $value;
	my $boolean;
	my $encode = 1;

	if ($ref eq "Gtk::Entry") {$value = $widget->get_text}
	elsif ($ref eq "Gtk::SpinButton") {$value = $widget->get_value_as_int}
	elsif ($ref eq "Gtk::CheckButton" || $ref eq "Gtk::ToggleButton") {if ($widget->get_active) {$value = 1} else {$value = 0}}
	$::settings->{$data} = $value;
		
	if ($widget eq $::widgets->{'audio_codec_entry'}) { set_bitrate() }
	if ($widget eq $::widgets->{'filesize_spin'}) { 
		$::widgets->{'filesize_spin'}->set_text("0") if $::widgets->{'filesize_spin'}->get_text!~/^\d+$/;
		set_bitrate() if $::dvd->{'valid'} eq 'yes';
	}
	if ($widget eq $::widgets->{'total_blocks_spin'}) { 
		on_filesize_changed();
		set_bitrate() if $::dvd->{'valid'} eq 'yes'
	}
	if ($widget eq $::widgets->{'tooltips_check'}) {
		if ($widget->get_active) {
			$::widgets->{'tooltips'}->enable
		} else {
			$::widgets->{'tooltips'}->disable
		}
	}
	if ($widget eq $::widgets->{'video_codec_entry'}) {
		if ($::widgets->{'video_codec_entry'}->get_text eq "lavc" 
				|| $::widgets->{'video_codec_entry'}->get_text eq "divx4"
				|| $::widgets->{'video_codec_entry'}->get_text eq "xvid") {
			$::widgets->{'video_options_entry'}->set_text($::settings->{$::widgets->{'video_codec_entry'}->get_text . "_options"});
			$encode = 1
		} else { $encode = 0 }
		$::widgets->{'video_bpp_entry'}->set_sensitive($encode);
		$::widgets->{'video_bpp_label'}->set_sensitive($encode);
		$::widgets->{'video_2pass_check'}->set_sensitive($encode);
		$::widgets->{'crop_enable_check'}->set_sensitive($encode);
		$::widgets->{'crop_detect_button'}->set_sensitive($encode);
		$::widgets->{'scale_enable_check'}->set_sensitive($encode);
		$::widgets->{'video_options_entry'}->set_sensitive($encode);
		$::widgets->{'video_bitrate_entry'}->set_sensitive($encode);
		$::widgets->{'video_options_label'}->set_sensitive($encode);
		$::widgets->{'video_bitrate_label'}->set_sensitive($encode);
		$::widgets->{'vop_filters_enable_check'}->set_sensitive($encode);
		$::widgets->{'vop_filters_entry'}->set_sensitive($encode);
	}
	if ($widget eq $::widgets->{'crop_enable_check'} || $widget eq $::widgets->{'video_codec_entry'}) {
		$boolean = $::widgets->{'crop_enable_check'}->get_active && $encode;
		$::widgets->{'crop_width_spin'}->set_sensitive($boolean);
		$::widgets->{'crop_width_label'}->set_sensitive($boolean);
		$::widgets->{'crop_height_spin'}->set_sensitive($boolean);
		$::widgets->{'crop_height_label'}->set_sensitive($boolean);
		$::widgets->{'crop_vertical_spin'}->set_sensitive($boolean);
		$::widgets->{'crop_vertical_label'}->set_sensitive($boolean);
		$::widgets->{'crop_horizontal_spin'}->set_sensitive($boolean);
		$::widgets->{'crop_horizontal_label'}->set_sensitive($boolean);
		$::widgets->{'crop_detect_button'}->set_sensitive($boolean);
	}
	if ($widget eq $::widgets->{'scale_enable_check'} || $widget eq $::widgets->{'video_codec_entry'}) {
		$boolean = $::widgets->{'scale_enable_check'}->get_active && $encode;
		$::widgets->{'scale_width_spin'}->set_sensitive($boolean);
		$::widgets->{'scale_width_label'}->set_sensitive($boolean);
		$::widgets->{'scale_height_spin'}->set_sensitive($boolean);
		$::widgets->{'scale_height_label'}->set_sensitive($boolean);
		$::widgets->{'scale_height_estimate_entry'}->set_sensitive($boolean);
		$::widgets->{'scale_lock_check'}->set_sensitive($boolean);
	}
	if ($widget eq $::widgets->{'vop_filters_enable_check'} || $widget eq $::widgets->{'video_codec_entry'}) {
		$boolean = $::widgets->{'vop_filters_enable_check'}->get_active && $encode;
		$::widgets->{'vop_filters_entry'}->set_sensitive($boolean);
	}
	if ($widget eq $::widgets->{'audio_codec_entry'}) {
		if ($::widgets->{'audio_codec_entry'}->get_text eq "mp3lame") {
			$boolean = 1;
			$::widgets->{'audio_options_entry'}->set_text($::settings->{$::widgets->{'audio_codec_entry'}->get_text . "_options"});		
		} else { $boolean = 0 }
		$::widgets->{'audio_options_entry'}->set_sensitive($boolean);
		$::widgets->{'audio_options_label'}->set_sensitive($boolean);
	}
	if ($widget eq $::widgets->{'video_options_entry'}) {
		$::settings->{$::widgets->{'video_codec_entry'}->get_text . "_options"} = $::widgets->{'video_options_entry'}->get_text;
	}
	if ($widget eq $::widgets->{'audio_options_entry'}) {
		$::settings->{$::widgets->{'audio_codec_entry'}->get_text . "_options"} = $::widgets->{'audio_options_entry'}->get_text;
		set_bitrate();
	}
	if ($widget eq $::widgets->{'scale_width_spin'} || $widget eq $::widgets->{'scale_height_spin'} ||
	  $widget eq $::widgets->{'crop_width_spin'} || $widget eq $::widgets->{'crop_height_spin'} ||
		$widget eq $::widgets->{'crop_horizontal_spin'} || $widget eq $::widgets->{'crop_vertical_spin'}) { 
		set_bpp(); 
	}
	if ($widget eq $::widgets->{'info_field_entry'} && $value ne '') {
		$::widgets->{'info_name_entry'}->hide;
		$::widgets->{'info_artist_entry'}->hide;
		$::widgets->{'info_genre_entry'}->hide;
		$::widgets->{'info_subject_entry'}->hide;
		$::widgets->{'info_comment_entry'}->hide;
		$::widgets->{'info_copyright_entry'}->hide;
		$::widgets->{'info_' . $value . '_entry'}->show;
	}
	if ($widget eq $::widgets->{'directory_filename_entry'}) {
		my @filearray = split/\//, $value;
		$::widgets->{'filename_entry'}->set_text($filearray[-1]);
		$value =~ s/$filearray[-1]//;
		$::widgets->{'directory_entry'}->set_text($value);
		on_filesize_changed();
	}
	if ($widget eq $::widgets->{'filename_entry'} || $widget eq $::widgets->{'directory_entry'}) {
		$::widgets->{'directory_filename_entry'}->set_text($::settings->{'directory'} . $::settings->{'filename'})
			if $::widgets->{'directory_filename_entry'}->get_text ne $::settings->{'directory'} . $::settings->{'filename'};
		set_warning_text ($::widgets->{'directory_filename_entry'}, -w $value ? 0 : 1) if $widget eq $::widgets->{'directory_entry'}
	}
	if ($widget eq $::widgets->{'crop_width_spin'}) {
		my $wo = $::dvd->{'track'}[$::settings->{'selected_track'}]->{'vts'}->{'width'};
		$::widgets->{'crop_horizontal_spin'}->set_adjustment(new Gtk::Adjustment(0, 0, $wo - $value, 1, 50, 200));
		set_bpp();
	}
	if ($widget eq $::widgets->{'crop_horizontal_spin'}) {
		my $wo = $::dvd->{'track'}[$::settings->{'selected_track'}]->{'vts'}->{'width'};
		$::widgets->{'crop_width_spin'}->set_adjustment(new Gtk::Adjustment(0, 0, $wo - $value, 1, 50, 200));
		set_bpp();
	}
	if ($widget eq $::widgets->{'crop_height_spin'}) {
		my $wo = $::dvd->{'track'}[$::settings->{'selected_track'}]->{'vts'}->{'height'};
		$::widgets->{'crop_vertical_spin'}->set_adjustment(new Gtk::Adjustment(0, 0, $wo - $value, 1, 50, 200));
		set_bpp();
	}
	if ($widget eq $::widgets->{'crop_vertical_spin'}) {
		my $wo = $::dvd->{'track'}[$::settings->{'selected_track'}]->{'vts'}->{'height'};
		$::widgets->{'crop_height_spin'}->set_adjustment(new Gtk::Adjustment(0, 0, $wo - $value, 1, 50, 200));
		set_bpp();
	}
	if ($widget eq $::widgets->{'cache_directory_entry'}) {
		set_warning_text ($widget, -w $value ? 0 : 1)
	}
	if ($widget eq $::widgets->{'dvd_device_entry'}) {
		set_warning_text ($widget, -r $value ? 0 : 1)
	}
	1;
}

sub on_save_settings_button_clicked
{
	message("Settings saved");
	$::settings->save_settings;
}

sub on_restore_defaults_button_clicked
{
	$::settings->restore_settings;
	load_settings_to_interface();
}

sub progress_dialog_show
{
	if ($::widgets->{'cache_menc_vbox'}->parent == $::widgets->{'right_vbox'}) {
		$::widgets->{'acidrip'}->hide;
		$::widgets->{'right_vbox'}->remove($::widgets->{'cache_menc_vbox'});
		$::widgets->{'progress_vbox'}->add($::widgets->{'cache_menc_vbox'});
		$::widgets->{'progress_dialog'}->show;
	}
}

sub progress_dialog_hide
{
	print "hello\n";
	if ($::widgets->{'cache_menc_vbox'}->parent == $::widgets->{'progress_vbox'}) {
		$::widgets->{'progress_dialog'}->hide;
		$::widgets->{'progress_vbox'}->remove($::widgets->{'cache_menc_vbox'});
		$::widgets->{'right_vbox'}->pack_start($::widgets->{'cache_menc_vbox'}, 0, 1, 0);
		$::widgets->{'acidrip'}->show;
	}
}

sub on_start_button_clicked
{
	my @cache_files = [];
	while (Gtk->events_pending()) { Gtk->main_iteration() };

	if ( -e $::settings->{'filename'} && ! $::settings->{'overwrite'}) {
		message("File already exists, refusing to overwrite");
		return -1;
	}
	if ( ( ! $::settings->{'enough_space'} ) && $::settings->{'enforce_space'}) {
		message("Not enough space in output directory!");
		return -1;
	}

	progress_dialog_show if $::settings->{'compact'};

	# cache if desired
	if ($::settings->{'cache'}) {
		#$::widgets->{'cache_frame'}->show;
		CACHE:
		for (1..$::settings->{'total_blocks'}) {		
			$::settings->{'this_block'} = $_ if $::settings->{'total_blocks'} > 1;
			my $cache = get_command('cache');
			message("Running $cache");
			message("Caching title...");
			@cache_files = (@cache_files, cache ($cache));
			last CACHE if $?
		}
		#$::widgets->{'cache_frame'}->hide;
		$::settings->{'this_block'} = 0;
		return if $?;
		system "eject $::settings->{'dvd_device'}";
	}
	
	ENCODE:
	for (1..$::settings->{'total_blocks'}) {
		$::settings->{'this_block'} = $_ if $::settings->{'total_blocks'} > 1;
		my $menc;
		my $message = "Encoding film";
		$message .= ", Block " . $::settings->{'this_block'} . " of " . $::settings->{'total_blocks'} if $::settings->{'this_block'} > 0;

		if ($::settings->{'video_2pass'}) {

			$::settings->{'video_2pass'} = 1;
			$menc = get_command("mencoder");
			message("Running $menc");
			message($message .  ", 1st pass...");
			unlink "frameno.avi";
			encode($menc);
			last ENCODE if $?;

			$::settings->{'video_2pass'} = 2;
			$menc = get_command("mencoder");
			message("Running $menc\n");
			message($message . ", 2nd pass...");
			encode($menc) if !$?;
			last ENCODE if $?;
			unlink "divx2pass.log";

		} else { 
			$menc = get_command("mencoder");
			message("Running $menc");
			message($message . "...");
			encode($menc);
			last ENCODE if $?;
		}
	}
	
	progress_dialog_hide if $::settings->{'compact'};
	
	if ($::settings->{'cache'} && $::settings->{'del_cache'}) {
		message("Deleting cache files");
		unlink @cache_files;
	}
	
	message("Encoding finished, hope it worked...");
	$::settings->{'this_block'} = 0;
	set_bitrate();
}

sub on_stop_button_clicked
{
	if ( $::settings->{'mencoder_pid'} > 0 ) {
		kill 'INT', $::settings->{'mencoder_pid'};
		kill 'INT', $::settings->{'mencoder_pid'} + 1 
			if `ps $::settings->{'mencoder_pid'} + 1 | grep mencoder` || `ps $::settings->{'mencoder_pid'} + 1 | grep mplayer`;
		message("Encoding stopped.");
		$::settings->{'mencoder_pid'} = -1;
	} else {
		message("Nothing to stop!")
	}
}

sub on_mencoder_output_show_button_clicked
{
	$::widgets->{'mencoder_output_frame'}->show_all;
	$::widgets->{'mencoder_output_show_button'}->hide;
}		
	
sub on_mencoder_output_hide_button_clicked
{
	$::widgets->{'mencoder_output_show_button'}->show;
	$::widgets->{'mencoder_output_frame'}->hide;
}	

sub on_mencoder_output_save_button_clicked
{
	if (open (LOGFILE, '>', "$ENV{HOME}/acidrip.log"))
	{
		print LOGFILE $::widgets->{'mencoder_output_text'}->get_chars(0, -1);
		close LOGFILE;
		message("Output log saved to $ENV{HOME}/acidrip.log")
	} else {
		message("Error opening output file")
	}
}

sub on_mencoder_output_saveall_button_clicked
{
	$::widgets->{'mencoder_output_save_button'}->set_sensitive(! $::widgets->{'mencoder_output_saveall_button'}->get_active);
	if ($::widgets->{'mencoder_output_saveall_button'}->get_active) {
		open (LOGFILE, '>', "$ENV{HOME}/acidrip.log");
		print LOGFILE $::widgets->{'mencoder_output_text'}->get_chars(0, -1);
	} else { close LOGFILE }
}

sub on_mencoder_output_clear_button_clicked
{
	message("Log information cleared");
	$::widgets->{'mencoder_output_text'}->delete_text(0,-1);
}

sub on_view_button_clicked
{
	#fork proofing
	open STDIN,  '/dev/null' or message("Can't read /dev/null: $!. May fail if in background");
	
	message("Playing back encoded track");
	my $mplayer_output;
	open (VIEW, get_command("view") . " 2>&1 |") || (message("mplayer preview failed, is it installed?"));
	while (<VIEW>) { 
		$mplayer_output .= $_; 
		while (Gtk->events_pending()) {Gtk->main_iteration()}
	}
	message($mplayer_output);
	message("Playing back encoded track... Finished");
}

sub on_read_dvd_button_clicked
{ 
  my ( $button, $widgets ) = @_;
  my $dvd = $::dvd;
  
	$::settings->{'selected_chapters_start'} = 0;
	$::settings->{'selected_chapters_end'} = 0;
	
  if (defined $widgets->{'root_tree_item'})
  {
		message("Reloading DVD... please wait");
    $widgets->{'root_tree_item'}->remove_subtree;
  } else {
		message("Loading DVD... please wait");
    $widgets->{'root_tree_item'} = new_with_label Gtk::TreeItem('This disc');
    $widgets->{'root_tree_item'}->show;
    $widgets->{'track_items'} = [];
    $widgets->{'chapter_tree'} = [];
		$widgets->{'chapter_items'} = [];
    $widgets->{'dvd_tree'}->append($widgets->{'root_tree_item'});
  }
	
	while (Gtk->events_pending()) {Gtk->main_iteration()}

  $widgets->{'root_tree_item'}->set_subtree($widgets->{'track_tree'});
  message($dvd->read_disc($::settings->{'lsdvd'}, $::settings->{'dvd_device'}));

	if ($dvd->{'valid'} eq 'yes')
  {
		message("DVD successfully read");
    my $nr_of_tracks = scalar @{$dvd->{'track'}} - 1 ;

    for my $this_track (1..$nr_of_tracks)
    {
      my $chapters = scalar @{$dvd->{'track'}[$this_track]->{'chapter'}} - 1;
      my $length = $dvd->get_length($this_track);
      $widgets->{'track_items'}[$this_track] = new_with_label Gtk::TreeItem(join(": ", $this_track, $length));
      $widgets->{'chapter_tree'}[$this_track] = new Gtk::Tree;

      $widgets->{'track_tree'}->append($widgets->{'track_items'}[$this_track]);
      $widgets->{'track_items'}[$this_track]->set_subtree($widgets->{'chapter_tree'}[$this_track]);
      $widgets->{'track_items'}[$this_track]->show;

			$widgets->{'chapter_items'}[$this_track] = [];

			for my $this_chapter (1..$chapters)
      {
        $widgets->{'chapter_items'}[$this_track][$this_chapter] = new_with_label Gtk::TreeItem(join(": ", "Chapter $this_chapter", 
					$dvd->{'track'}[$this_track]->{'chapter'}[$this_chapter]->{'length'}));
        $widgets->{'chapter_tree'}[$this_track]->append($widgets->{'chapter_items'}[$this_track][$this_chapter]);

				$widgets->{'chapter_items'}[$this_track][$this_chapter]->signal_connect('deselect', \&on_chapter_deselect, $this_chapter, $this_track);
				$widgets->{'chapter_items'}[$this_track][$this_chapter]->signal_connect('select', \&on_chapter_select, $this_chapter, $this_track);
        $widgets->{'chapter_items'}[$this_track][$this_chapter]->show;
      }
    }
	}	else {
		message("Error loading DVD");
		my $not_found_item = new_with_label Gtk::TreeItem('Not found');
		$widgets->{'track_tree'}->append($not_found_item);
		$not_found_item->show;
	}

	$widgets->{'root_tree_item'}->expand;
	$widgets->{'track_tree'}->select_item($dvd->get_main_title - 1);

	my $name=$::dvd->{'title'};
	$name=~tr/A-Z/a-z/;

	set_setting('directory_filename', $::settings->{'directory'} . $name);
}	

1;
