#
#   Makefile.PL for acidrip
#    - version 0.8
#
require 5.000;
use ExtUtils::MakeMaker;
use strict;

#--- Configuration section ---

my @programs_to_install = qw(acidrip);

my @need_perl_modules = (
    # Check for Gtk::Types rather than the Gtk supermodule
    #   this avoids dumping MakeMaker
    {'name'     => 'Gtk',
    'test'      => 'Gtk::Types',
    'version'   => '0.7000',
    'reason'    => "implements the perl bindings to Gtk+.\n".
                    "The module is called Gtk-Perl on CPAN or ".
                    "module gnome-perl in the Gnome CVS.\n".
		    "In the future acidrip will be able to run from a command line, so this will not be mandatory.\n".
		    "This has not been implemented yet though."
    },
    );
#--- End Configuration - You should not have to change anything below this line

# Allow us to suppress all program installation with the -n (library only)
# option.  This is for those that don't want to mess with the configuration
# section of this file.
use Getopt::Std;
use vars qw($opt_n);
unless (getopts("n")) {
    die "Usage: $0 [-n]\n";
}
@programs_to_install = () if $opt_n;

# Check for non-standard modules that are used by this library.
$| = 1; # autoflush on
my $missing_modules = 0;

foreach my $mod (@need_perl_modules) {
    print "Checking for $mod->{'name'}..";
    eval "require $mod->{'test'}";
    if ($@) {
        $missing_modules++;
        print " failed\n";
        print   "-------------------------------------------------------".
                "\n$@\n",
                "$mod->{'name'} is needed, it $mod->{'reason'}\n",
                "We need at least version $mod->{'version'}\n".
                "-------------------------------------------------------\n";
        sleep(2);  # Don't hurry too much
    } else {
        print " ok\n";
    }
}

#--------------------------------------
print "-------------------------------------------------------
The missing modules can be obtained from CPAN. Visit
<URL:http://www.perl.com/CPAN/> to find a CPAN site near you.
-------------------------------------------------------\n\n"
     if $missing_modules;

#--------------------------------------
# Last of all generate the Makefile
WriteMakefile(
    'DISTNAME'     => 'acidrip',
    'NAME'         => 'AcidRip',
    'VERSION_FROM' => 'acidrip',
    'EXE_FILES'    => [ @programs_to_install ],
    'clean'        => { FILES => '' },
    'dist'         => { COMPRESS => 'gzip', SUFFIX => 'gz' }
);

package MY;

# Pass Glade-Perl version number to pod2man
sub manifypods
{
    my $self = shift;
    my $ver = $self->{'VERSION'} || "";
    local($_) = $self->SUPER::manifypods(@_);
    s/pod2mans*$/pod2man --release acidrip-$ver/m;
    $_;
}

print "\nRunning additional test to be sure acidrip will work\n";

print "test for lsdvd: "; die "Not Found!\n\n***************\n" .
  "AcidRip needs my little app \"lsdvd\" in order to see what is on your dvd\'s\n" .
  "Download it and install it (and libdvdread) before trying to use AcidRip.\n" .
  "If it\'s simply not on your path then set the full path inside AcidRip.\n" .
  "***************\n\n" if ( system "lsdvd -h 2> /dev/null" ); print "found\n";

print "test for MPlayer: "; die "Not Found!\n\n***************\n" .
  "AcidRip is based around the MPlayer video player. it has not been found on\n" .
  "your current path. Install it from http://mplayerhq/hu see INSTALL.html or\n" .
  "INSTALL for installation instructions.\n" .
  "***************\n\n" if ( system "mplayer 2&>1 > /dev/null" ); print "found\n";

print "test for MEncoder: "; die "Not Found!\n\n***************\n" .
  "AcidRip uses MEncoder, which is part of MPlayer video player package. This\n" .
  "was not found. Install it from http://mplayerhq/hu see INSTALL.html or\n" .
  "INSTALL for installation instructions.\n" .
  "***************\n\n" if ( system "mencoder -ovc help 2> /dev/null | grep MEncoder > /dev/null" ); print "found\n";

print "test for MP3 support in mencoder: "; die "Not Found!\n\n***************\n" .
  "MEncoder was found, however it does not have MP3 support built into it.\n" .
  "If you DO want MP3 support you'll need to recompile mplayer with mp3 support\n" .
  "See INSTALL or INSTALL.html for more information\n" .
  "***************\n\n" if ( system "mencoder -oac help 2> /dev/null | grep mp3lame > /dev/null" ); print "found\n";

print "Everything looks good here...\n"; 

print "\nPlease send any feedback to me, I *Really* want to know what you think of it.\nMaybe then I can make it suck less. acid_kewpie\@users.sourceforge.net\n\n"; 
exit(0);

